#include "syscall.h"
#include "io_lib.h"

/*
  This program will do error-testing of file-based I/O in nachos.
  The goal is to catch as many boundary cases as possible
 */


void main() {
  int status;
  OpenFileId fd, fd1;
  char buf[128];

  /* Try to create a file with a zero length filename */  
  print("Trying to create file with zero length name... ");
  status = Create("");
  if(status == 0 ) {
    print("succeeded (it shouldn't)\n");
    Exit(1);
  } else {
    print("failed (it should)\n");
  }

  /* Try to read from a bad file-id */
  print("Trying to read using a bad file descriptor... ");
  status = Read(buf, 32, -1 );
  if( status >= 0 ) {
    print("succeeded (it shouldn't)\n");
    Exit(1);
  } else {
    print("failed (it should)\n");
  }

  /* Try to write to a bad file-id */
  print("Trying to write using a bad file descriptor... ");
  status = Write("Hello World\n", 12, -12 );
  if( status >= 0 ) {
    print("succeeded (it shouldn't)\n");
    Exit(1);
  } else {
    print("failed (it should)\n");
  }

  /* Create a file to use in other tests */
  print("Creating test.txt... ");
  status = Create("test.txt");
  if( status != 0 ) {
    print("failed\n");
    Exit(1);
  } else {
    print("OK\n");
  }

  /* Open the file */
  print("Opening test.txt... ");
  fd = Open("test.txt", "rw");
  if( fd < 0 ) {
    print("failed! (it shouldn't)\n");
    Exit(1);
  } else {
    print("succeeded! (it should)\n");
  }

  /* Open it again */
  print("Opening test.txt a second time... ");
  fd1 = Open("test.txt", "rw");
  if( fd1 < 0 ) {
    print("failed! (it shouldn't)\n");
    Exit(1);
  } else {
    print("succeeded! (it should)\n");
  }

  /* Try to read/write a negative number of bytes */
  print("Trying to read -5 bytes from test.txt... ");
  status = Read(buf, -5, fd );
  if( status >= 0 ) {
    print("succeeded (it shouldn't)\n");
    Exit(1);
  } else {
    print("failed (it should)\n");
  }

  print("Trying to write -5 bytes to test.txt... ");
  status = Write(buf, -5, fd );
  if( status >= 0 ) {
    print("succeeded (it shouldn't)\n");
    Exit(1);
  } else {
    print("failed (it should)\n");
  }

  print("Writing data to test.txt... ");
  status = Write("Hello World\n", 12, fd );
  if( status != 12 ) {
    print("failed (it shouldn't)\n");
    Exit(1);
  } else {
    print("succeeded (it should)\n");
  }


  /* Now close the file */  
  print("Closing test.txt (first fd)... ");
  status = Close(fd);
  if( status < 0 ) {
    print("failed (it shouldn't)\n");
    Exit(1);
  } else {
    print("succeeded\n");
  }  
 
  /* Should still be able to use 2nd fd */
  print("Reading data from test.txt using 2nd fd... ");
  status = Read(buf, 12, fd1 );
  if (status != 12 ) {
    print("failed (it shouldn't)\n");
    Exit(1);  
  } else {
    print("succeeded\n");
  } 

  /* Should not be able to read on closed fd */
  print("Trying to read using closed (first) fd... ");
  status = Read(buf, 12, fd );
  if( status > 0 ) {
    print("succeeded (it shouldn't)\n");
    Exit(1);
  } else {
    print("failed (it should)\n");
  }

  /* Close 2nd fd */
  print("Closing test.txt (second fd)... ");
  status = Close(fd1);
  if( status < 0 ) {
    print("failed (it shouldn't)\n");
    Exit(1);
  } else {
    print("succeeded\n");
  }  
 
  /* Try an extra close */
  print("Trying an extra close on first fd... ");
  status = Close(fd);
  if( status < 0 ) {
    print("failed (it should)\n");
  } else {
    print("succeeded\n");
    Exit(1);
  }  

  print("TEST COMPLETED SUCCESSFULLY\n");

}
