/* Start.s 
 *	Assembly language assist for user programs running on top of Nachos.
 *
 *	Since we don't want to pull in the entire C library, we define
 *	what we need for a user program here, namely Start and the system
 *	calls.
 */

#define IN_ASM
#include "syscall.h"

        .text   
        .align  2

/* -------------------------------------------------------------
 * __start
 *	Initialize running a C program, by calling "main". 
 *
 * 	NOTE: This has to be first, so that it gets loaded at location 0.
 *	The Nachos kernel always starts a program by jumping to location 0.
 * -------------------------------------------------------------
 */

	.globl __start
	.ent	__start
__start:
	jal	main
	move	$4,$0		
	jal	Exit	 /* if we return from main, exit(0) */
	.end __start

/* -------------------------------------------------------------
 * System call stubs:
 *	Assembly language assist to make system calls to the Nachos kernel.
 *	There is one stub per system call, that places the code for the
 *	system call into register r2, and leaves the arguments to the
 *	system call alone (in other words, arg1 is in r4, arg2 is 
 *	in r5, arg3 is in r6, arg4 is in r7)
 *
 * 	The return value is in r2. This follows the standard C calling
 * 	convention on the MIPS.
 * -------------------------------------------------------------
 */

/****************** Processes and Threads *********************/
	.globl Halt
	.ent	Halt
Halt:
	addiu $2,$0,SC_Halt
	syscall
	j	$31
	.end Halt

	.globl Exit
	.ent	Exit
Exit:
	addiu $2,$0,SC_Exit
	syscall
	j	$31
	.end Exit

	.globl Exec
	.ent	Exec
Exec:
	addiu $2,$0,SC_Exec
	syscall
	j	$31
	.end Exec


        .globl ExecV
        .ent    ExecV
ExecV:
        addiu $2,$0,SC_ExecV
        syscall
        j       $31
        .end ExecV

	.globl Join
	.ent	Join
Join:
	addiu $2,$0,SC_Join
	syscall
	j	$31
	.end Join

        .globl ThreadFork
        .ent    ThreadFork
ThreadFork:
        addiu $2,$0,SC_ThreadFork
        syscall
        j       $31
        .end ThreadFork

        .globl ThreadYield
        .ent    ThreadYield
ThreadYield:
        addiu $2,$0,SC_ThreadYield
        syscall
        j       $31
        .end ThreadYield

        .globl ThreadExit
        .ent    ThreadExit
ThreadExit:
        addiu $2,$0,SC_ThreadExit
        syscall
        j       $31
        .end ThreadExit

	
        .globl Suspend
        .ent    Suspend
Suspend:
        addiu $2,$0,SC_Suspend
        syscall
        j       $31
        .end Suspend

        .globl Resume
        .ent    Resume
Resume:
        addiu $2,$0,SC_Resume
        syscall
        j       $31
        .end Resume

        .globl SetPriority
        .ent    SetPriority
SetPriority:
        addiu $2,$0,SC_SetPriority
        syscall
        j       $31
        .end SetPriority

        .globl GetPriority
        .ent    GetPriority
GetPriority:
        addiu $2,$0,SC_GetPriority
        syscall
        j       $31
        .end GetPriority

        .globl GetId
        .ent    GetId
GetId:
        addiu $2,$0,SC_GetId
        syscall
        j       $31
        .end GetId
	
        .globl GetParentId
        .ent    GetParentId
GetParentId:
        addiu $2,$0,SC_GetParentId
        syscall
        j       $31
        .end GetParentId
	
/****************** Files and File System *********************/
	.globl Create
	.ent	Create
Create:
	addiu $2,$0,SC_Create
	syscall
	j	$31
	.end Create

	.globl Remove
	.ent	Remove
Remove:
	addiu $2,$0,SC_Remove
	syscall
	j	$31
	.end Remove

	.globl Open
	.ent	Open
Open:
	addiu $2,$0,SC_Open
	syscall
	j	$31
	.end Open

	.globl Read
	.ent	Read
Read:
	addiu $2,$0,SC_Read
	syscall
	j	$31
	.end Read

	.globl Write
	.ent	Write
Write:
	addiu $2,$0,SC_Write
	syscall
	j	$31
	.end Write
	
	.globl Close
	.ent	Close
Close:
	addiu $2,$0,SC_Close
	syscall
	j	$31
	.end Close

        .globl Seek
	.ent   Seek
Seek:
	 addiu $2,$0,SC_Seek
	 syscall
	 j       $31
	 .end Seek

        .globl CreateDir
        .ent    CreateDir
CreateDir:
        addiu $2,$0,SC_CreateDir
        syscall
        j       $31
        .end CreateDir
	
        .globl RemoveDir
        .ent    RemoveDir
RemoveDir:
        addiu $2,$0,SC_RemoveDir
        syscall
        j       $31
        .end RemoveDir

	.globl GetFileLen
	.ent    GetFileLen
GetFileLen:
	addiu $2,$0,SC_GetFileLen
	syscall
	j       $31
	.end GetFileLen

/************************* Locks **********************************/

        .globl LockOpen
        .ent    LockOpen
LockOpen:
        addiu $2,$0,SC_LockOpen
        syscall
        j       $31
        .end LockOpen

        .globl LockClose
        .ent    LockClose
LockClose:
        addiu $2,$0,SC_LockClose
        syscall
        j       $31
        .end LockClose

        .globl LockAcquire
        .ent    LockAcquire
LockAcquire:
        addiu $2,$0,SC_LockAcquire
        syscall
        j       $31
        .end LockAcquire

        .globl LockRelease
        .ent    LockRelease
LockRelease:
        addiu $2,$0,SC_LockRelease
        syscall
        j       $31
        .end LockRelease


/************************* Shared Memory **********************************/

        .globl SharedMemOpen
        .ent    SharedMemOpen
SharedMemOpen:
        addiu $2,$0,SC_SharedMemOpen
        syscall
        j       $31
        .end SharedMemOpen

        .globl SharedMemClose
        .ent    SharedMemClose
SharedMemClose:
        addiu $2,$0,SC_SharedMemClose
        syscall
        j       $31
        .end SharedMemClose


/************************* Pipes **********************************/
        .globl CreatePipe
        .ent    CreatePipe
CreatePipe:
        addiu $2,$0,SC_CreatePipe
        syscall
        j       $31
        .end CreatePipe

        .globl RemovePipe
        .ent    RemovePipe
RemovePipe:
        addiu $2,$0,SC_RemovePipe
        syscall
        j       $31
        .end RemovePipe


/******************************************************************/

/* dummy function to keep gcc happy */
        .globl  __main
        .ent    __main
__main:
        j       $31
        .end    __main



