
/*
 * Title   : testutils
 * Author  : Tim Brecht
 * Date    : Thu Dec 23 21:19:00 EST 2004
 *
 * Some simple utilities that can be helpful for testing.
 * Feel free to add/modify this code to suit your needs.
 * NOTE: this hasn't been fully tested.
 *
 */

#include "testutils.h"

/* Used to track the number of tests and failures */
static int test_num = 0;
static int num_failures = 0;
static int verbose = 0;

/* Use to print the source file, function and line */
static void
print_location(char *file, char *func, int line)
{
    print("   ");
    print(file);
    print(" : function = ");
    print(func);
    print(", line = ");
    printInt(line);
    print("\n");
}

/* Check if val is equal to expected_val, if not it is an error */
void
test_equal(int val, int expected_val, char *str, 
            char *file, char *func, int line)
{
    int failed = 0;
    char *out = "SUCCESS";

    test_num++;

    if (val != expected_val) {
	num_failures++;
        failed = 1;
        out = "FAILURE";
    }

    if (failed || verbose) {
	print(out);
	print(" ON TEST = ");
	printInt(test_num);
	print(" : Got ");
	printInt(val);
	print(" : expected ");
	printInt(expected_val);
	print("\n");
        if (failed) {
	    print("   ");
	    print(str);
	    print("\n");
        }
	print_location(file, func, line);
    }
} /* test_equal */

/* Check if val is > 0, if not it is an error */
void
test_positive(int val, char *str,
              char *file, char *func, int line)
{
    int failed = 0;
    char *out = "SUCCESS";

    test_num++;

    if (val <= 0) {
	num_failures++;
        failed = 1;
        out = "FAILURE";
    }

    if (failed || verbose) {
        print(out);
	print(" ON TEST = ");
	printInt(test_num);
	print(" : Got ");
	printInt(val);
	print(" : expected Positive Value\n");
        if (failed) {
	    print("   ");
	    print(str);
	    print("\n");
        }
	print_location(file, func, line);
    }
}

/* Check if val is < 0, if not it is an error */
void
test_negative(int val, char *str,
               char *file, char *func, int line)
{
    int failed = 0;
    char *out = "SUCCESS";

    test_num++;

    if (val >= 0) {
	num_failures++;
        failed = 1;
	out = "FAILURE";
    }

    if (failed || verbose) {
        print(out);
	print(" ON TEST = ");
	printInt(test_num);
	print(" : Got ");
	printInt(val);
	print(" : expected Negative Value\n");
        if (failed) {
	    print("   ");
	    print(str);
	    print("\n");
        }
	print_location(file, func, line);
    }
}

/* Check if val1 is not equal to val2, if not it is an error */
void
test_not_equal(int val1, int val2, char *str,
                char *file, char *func, int line)
{
    int failed = 0;
    char *out = "SUCCESS";

    test_num++;

    if (val1 == val2) {
	num_failures++;
        failed = 1;
	out = "FAILURE";
    }

    if (failed || verbose) {
        print(out);
	print(" ON TEST = ");
	printInt(test_num);
        print(" Got ");
        printInt(val1);
	print(" Expected anything but ");
	printInt(val2);
	print("\n");
        if (failed) {
	    print("    ");
	    print(str);
	    print("\n");
        }
	print_location(file, func, line);
    }
}

/* Reset the statistic counters */
void
test_reset_stats()
{
    num_failures = 0;
    test_num = 0;
}

void
test_verbose_on()
{
    print("TEST VERBOSE ON\n");
    verbose = 1;
}

void
test_verbose_off()
{
    print("TEST VERBOSE OFF\n");
    verbose = 0;
}

void
test_print_stats(char *file, char* func, int line)
{
    print("TEST STATS for ");
    print(file);
    print(" : from function = ");
    print(func);
    print(", line = ");
    printInt(line);
    print("\n");
    print("    Number of failures = ");
    printInt(num_failures);
    print("   Number of Tests = ");
    printInt(test_num);
    print("\n");
}
