// addrspace.h 
//	Manage process address spaces.
//
// Copyright (c) 1992-1996 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef ADDRSPACE_H
#define ADDRSPACE_H

#include "copyright.h"

class OpenFile;
class Table;
class TranslationEntry;
class SynchBitmap;


#define UserStackSize		1024 	// increase this as necessary!

//  this is the maximum number of open files per process
//  file identifiers are guaranteed to be greater than one (because
//  we reserve fileIDs 0 and 1 for the console) and less than
//  MaxOpenFiles+2

#define MaxOpenFiles            32      // maximum number of open files
                                        // per address space

// Maximum length of string for UserStringToKernel

#define  MaxStringArgLength 256

enum TranslationStatus {
  OK,
  PageFault,
  ProtectionError,
  BusError,          // invalid physical address
  AddressError       // invalid virtual address
};

class AddrSpace {
  public:
    // Create an address space, initializing it with the program
    // stored in the file "executable"
    // "map" is a bitmap that can be used to find free memory frames
    // "myid" is the unique identifier for the new address space
    AddrSpace(OpenFile *executable, SynchBitmap *map,int myid);

    ~AddrSpace();			// De-allocate an address space

    void InitRegisters();          // Initialize user-level CPU registers

    void SaveState();			// Save/restore address space-specific
    void RestoreState();		// info on a context switch 

                                        // Utility routine to map virtual
                                        // addresses to physical addresses
    TranslationStatus
      Translate(int vaddr,int *paddr, bool write);

    TranslationStatus
      ReadMem(int addr, int size, int *value);  // read from virtual address

    TranslationStatus
      WriteMem(int addr, int size, int value); // write to virtual address


    int  AddOpenFile(OpenFile *file);   // Add a new open file to the
                                        //  open file list.  Returns an
                                        //  integer file identifier
    OpenFile* DeleteOpenFile(int fId);  // Remove file with the specified
                                        //  identifier from the open file list
                                        // and return the open file pointer
    OpenFile* GetOpenFile(int fId);   // Return the open file with the
                                        //  specified identifier

    // utility functions for copying data between address space
    // and kernel data structures
    int UserStringToKernel(int vaddr, char* buf);    
    int UserBufToKernel(int vaddr, int length, char* buf);
    int KernelToUserBuf(int vaddr, int length, char* buf);
    
    int  id;                        // a unique identifier for
                                        // this address space.
                                        // a negative value indicates
                                        //  an invalid address space that
                                        // could not be properly initialized
                                        // because of insufficient main
                                        // memory

  private:
    TranslationEntry *pageTable;	
    int numPages;		// Number of pages in the virtual 
					// address space

    Table* openFiles;                   // An table of pointers
                                        // to open files associated
                                        //  with this address space
    SynchBitmap*  memMap;              // pointer to memory map

};

#endif // ADDRSPACE_H
