/* errno.h 

 **** NOTE: These are suggestions for possible error codes.
            These may or may not apply in different terms.
            You are not required to return these but you
            may find it can make testing much easier.

 This file defines some negative error codes to be returned by Nachos
 system calls.  The purpose of these codes is to help user programs
 identify the cause of the problem when a system call fails.

 Of course, for these to be useful, you must implement your new system
 calls in such a way that they return appropriate error codes when they
 encounter an error.

 You are also encouraged to add your own error codes
 but be sure to begin with the value -100 (as shown below).

*/

#ifndef ERRNO_H
#define ERRNO_H

/* Many of these names are examples from UNIX/Linux 
 * They have been prefixed with NACHOS_ to avoid name clashes.
 */ 

#define	NACHOS_EPERM		(-10)	/* Operation not permitted */
#define	NACHOS_ENOENT		(-11)	/* No such file or directory */
#define	NACHOS_EBADID		(-12)	/* Bad identifier */
#define	NACHOS_EACCES		(-13)	/* Permission denied */
#define	NACHOS_EFAULT		(-14)	/* Bad address */
#define	NACHOS_EBUSY		(-15)	/* Device or resource busy */
#define	NACHOS_EEXIST		(-16)	/* File exists */
#define	NACHOS_EISDIR		(-17)	/* Is a directory */
#define	NACHOS_EINVAL		(-18)	/* Invalid argument */
#define	NACHOS_ENFILE		(-19)	/* File table overflow */
#define	NACHOS_EMFILE		(-20)	/* Too many open files */
#define	NACHOS_ENOSPC		(-21)	/* No space left on device */
#define	NACHOS_ENAMETOOLONG	(-22)	/* File name too long */
#define	NACHOS_ENOSYS		(-23)	/* Function not implemented */
#define	NACHOS_ENOTEMPTY	(-24)	/* Directory not empty */
#define	NACHOS_ENOTHELD 	(-25)	/* Not in critical section */ 
                                        /* lock not held */
#define	NACHOS_EALREADYHELD 	(-26)	/* Can't grab lock you are */
                                        /* already holding */
#define	NACHOS_ENOTEXCLUSIVE	(-27)	/* Lock is held by more than */
                                        /* one entity */
#define	NACHOS_ENOMEM       	(-28)	/* No memory available */
#define	NACHOS_EREACHEDLIMIT	(-29)	/* Available resource have */
                                        /* been consumed */

#define	FIRST_NEW_CODE  (-100)	/* Start your new codes at this value */

#define NACHOS_ECHANGE_ME       (-100)

#endif // ERRNO_H





