// synchconsole.h
//
// Synchronous console implemented using the asynchronous
// console provided by the machine.
//
//
// Copyright (c) 1992-1996 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef SYNCHCONSOLE_H
#define SYNCHCONSOLE_H

#include "copyright.h"
#include "console.h"

class Semaphore;

// The following two classes define synchronized input and output to
// a console device

class SynchConsoleInput : public CallBackObj {
  public:
    SynchConsoleInput(char *inputFile); // Initialize the console device
    ~SynchConsoleInput();		// Deallocate console device

    char GetChar();		// Read a character, waiting if necessary
    // Read "length" characters from input console into kernel buffer
    void ConsoleToKernel(int length, char* buf);
    
  private:
    ConsoleInput *consoleInput;	// the hardware keyboard
    Semaphore *inputReady;      // wait for input data to be available

    void CallBack();		// called when a keystroke is available
};

class SynchConsoleOutput : public CallBackObj {
  public:
    SynchConsoleOutput(char *outputFile); // Initialize the console device
    ~SynchConsoleOutput();

    void PutChar(char ch);	// Write a character, waiting if necessary
    // Write "length" characters from kernel buffer to output console
    void KernelToConsole(int length, char* buf);
    
  private:
    ConsoleOutput *consoleOutput;// the hardware display
    Semaphore *outputReady;	// wait for output to be ready

    void CallBack();		// called when more data can be written
};

#endif // SYNCHCONSOLE_H
