/*
 * Title   : example-test2
 * Author  : Tim Brecht
 * Date    : Sat Oct  2 19:08:17 EST 1999
 *
 * Some more example tests using Create, Open, and Close.
 * Assumes that files name FILE1 and FILE2 do not exist in current directory
 *
 * Modified: Thu Dec 23 17:15:50 EST 2004
 *   TBB - updated for Winter 2005 term.
 */

#include "testutils.h"
#include "syscall.h"
#include "errno.h"

/* Uncomment this if/when returning specific error codes using errno.h */
/*
#define USING_ERR_CODES
*/

/* This is a long name that we should be able to create */

#define LONGNAME \
"A_80_CHAR_NAME_AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"

/* Create a name that is long enough so that it fails */

#define REALLYLONGNAME \
"A_257_CHAR_NAME_AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" 

/* This is a name that doesn't exist so it opening it should fail */
#define BOGUS_NAME "ZZ12ZT"

int
main()
{
  OpenFileId f1, f2, f3;
  int i = 42;
  int j = -999;
  int rc = 0;
  int count = 0;

  /* Check that create succeeds */
  rc = Create("FILE1");
  TEST_EQUAL(rc, SUCCESS, "Unable to create FILE1");

  /* Check that create succeeds */
  rc = Create("FILE2");
  TEST_EQUAL(rc, SUCCESS, "Unable to create FILE2");

  /* check that we can open the same file many times */
  f1 = Open("FILE1", "rw");
  TEST_POSITIVE(f1, "Unable to open FILE1 first time");

  f2 = Open("FILE1", "rw");
  TEST_POSITIVE(f2, "Unable to open FILE1 second time");

  f3 = Open("FILE1", "rw");
  TEST_POSITIVE(f3, "Unable to open FILE1 third time");

  /* Check that f1 != f2 != f3 */
  TEST_NOT_EQUAL(f1, f2, "Using same OpenFileId for multiple opens f1 = f2");
  TEST_NOT_EQUAL(f2, f3, "Using same OpenFileId for multiple opens f2 = f3");

  rc = Close(f1);
  TEST_EQUAL(rc, SUCCESS, "Unable to close f1");
  rc = Close(f2);
  TEST_EQUAL(rc, SUCCESS, "Unable to close f2");
  rc = Close(f3);
  TEST_EQUAL(rc, SUCCESS, "Unable to close f3");

  /* Try writing to a close file should fail */
  rc = Write((char *) &i, sizeof(i), f1);
  TEST_NEGATIVE(rc, "Writing to closed file f1 should fail");
#ifdef USING_ERR_CODES
  TEST_EQUAL(rc, NACHOS_EINVAL, "Expected NACHOS_EINVAL when writing to closed file f1");
#endif /* USING_ERR_CODES */

  rc = Read((char *) &j, sizeof(j), f2);
  TEST_NEGATIVE(rc, "Read from closed file f2 should fail");
#ifdef USING_ERR_CODES
  TEST_EQUAL(rc, NACHOS_EINVAL, "Expected NACHOS_EINVAL when reading from closed file f2");
#endif /* USING_ERR_CODES */

  rc = Close(0xdeadbeef);
  TEST_NEGATIVE(rc, "Close on invalid file id didn't return error code");
#ifdef USING_ERR_CODES
  TEST_EQUAL(rc, NACHOS_EINVAL, "Expected NACHOS_EINVAL when closing invalid OpenFileId");
#endif /* USING_ERR_CODES */

  rc = Open(BOGUS_NAME, "rw");
  TEST_NEGATIVE(rc, "Open non-existant file returns incorrect value");
#ifdef USING_ERR_CODES
  TEST_EQUAL(rc, NACHOS_ENOENT, "Expected NACHOS_ENOENT when opening non existant file");
#endif /* USING_ERR_CODES */
  
  do {
    f1 = Open("FILE1", "rw");
    if (f1 >= 0) {
	  count++;
	}
  } while (f1 >= 0);

  TEST_NEGATIVE(f1, "Opening too many files doesn't return error code");
  TEST_POSITIVE(count, "Count of open files expected to be > 0");

  print("Number of files opened = ");
  printInt(count);
  print("\n");

#if 0
  /* Current implementation of open doesn't handle this error properly */
  rc = Open((char *) 0xffffffff, "rw");
  TEST_NEGATIVE(rc, "Open file using bad address doesn't return error code");
#ifdef USING_ERR_CODES
  TEST_EQUAL(rc, NACHOS_EFAULT, "Expected NACHOS_EFAULT for invalid address for filename");
#endif /* USING_ERR_CODES */
#endif

  rc = Create(LONGNAME);
  TEST_EQUAL(rc, SUCCESS, "Unable to open file with LONGNAME");
  
  rc = Create(REALLYLONGNAME);
  TEST_NEGATIVE(rc, "Expected error code when trying to open REALLYLONGNAME");
#ifdef USING_ERR_CODES
  TEST_EQUAL(rc, NACHOS_ENAMETOOLONG, "Expected NACHOS_ENAMETOOLONG when creating REALLYLONGNAME");
#endif /* USING_ERR_CODES */

  TEST_STATS();

  Halt();
}
